// 16:34

import { Connection } from "rabbitmq-client";

// Initialize:
const rabbit = new Connection("amqp://guest:guest@localhost:5672");
rabbit.on("error", (err) => {
  console.log("RabbitMQ connection error", err);
});
rabbit.on("connection", () => {
  console.log("Connection successfully (re)established");
});

const pub = rabbit.createPublisher({
  // Enable publish confirmations, similar to consumer acknowledgements
  confirm: true,
  // Enable retries
  maxAttempts: 2,
  // Optionally ensure the existence of an exchange before we use it
  exchanges: [{ exchange: "events", type: "topic" }],
});

setInterval(
  async () =>
    await pub.send(
      { exchange: "events", routingKey: "0" },
      { id: 1, name: "Alan Turing" }
    ),
  2000
);

async function onShutdown() {
  // Waits for pending confirmations and closes the underlying Channel
  await pub.close();
  await rabbit.close();
}

process.on("SIGINT", onShutdown);
process.on("SIGTERM", onShutdown);
