import rabbit, { Consumer } from "rabbitmq-stream-js-client";

const client = await rabbit.connect({
  hostname: "localhost",
  port: 5552,
  username: "guest",
  password: "guest",
  vhost: "/",
});

const messages = [
  "0::abc",
  "0::hello",
  "1::1", // ignored
  "0:sub:1",
  "1::2",
  "1:sub:2",
  "2::1",
  "3::1", // ignored
  "delay", // ignored
  "2:sub:3",
  "3::1",
  "3::2",
  "3::3",
  "3::4",
  "3::5",
  "3::6",
  "delay", // ignored
  "3::7",
  "3::8",
  "3:sub:4",
  "4:sub:5",
  "5:sub:6",
  "6:sub:7",
  "7:sub:8",
  "8:sub:9",
];

const count = messages.length - 4;

const received = new Set();

async function publishAll() {
  const streamSizeRetention = 5 * 1e9;
  await client.createStream({
    stream: "dynamic-test",
    arguments: { "max-length-bytes": streamSizeRetention },
  });

  const publisher = await client.declarePublisher(
    { stream: "dynamic-test" },
    (msg) => {
      // const topic = String.fromCharCode(msg.content[0]);
      //   console.log("Publish", topic);
      return msg.applicationProperties!.i as string;
    }
  );

  for (const msg of messages) {
    if (msg === "delay") {
      await new Promise((resolve) => setTimeout(resolve, 1000));
    } else {
      await publisher.send(Buffer.from(msg), {
        applicationProperties: { i: msg.split(":").shift()! },
      });
    }

    // await new Promise((resolve) => setTimeout(resolve, 1));
  }

  await publisher.close();
}

async function createListener() {
  let topics = ["0"];
  let subscriber: Consumer;

  let cur = 0;
  const fn = (i) => {
    return async (msg) => {
      if (i !== cur)
        return console.warn("Ignore message from old consumer", i, msg.offset);

      //   console.log(msg);
      const instruction = msg.content.toString();
      received.add(instruction);
      console.info(
        "Received",
        received.size,
        "out of",
        count,
        msg.offset,
        instruction
      );

      const [, task, arg] = instruction.split(":");
      if (task === "sub") {
        subscriber
          .close()
          .catch(() => console.error("Failed to close subscriber", i));

        const subCopy = subscriber;
        subCopy
          .close()
          .catch(() => console.error(`Failed to close a subscriber...`));

        topics.push(arg);

        console.info(
          "Rebuild consumer with new topic list...",
          msg.offset! + 1n
        );

        cur = i + 1;

        let values = [...topics];
        subscriber = await client.declareConsumer(
          {
            stream: "dynamic-test",
            offset: rabbit.Offset.offset(msg.offset! + 1n),
            filter: {
              values,
              postFilterFunc() {
                if (values.includes(msg.applicationProperties!.i as string))
                  return true;
                else return false;
              },
              matchUnfiltered: false,
            },
          },
          fn(i + 1)
        );
      }
    };
  };

  let values = [...topics];
  subscriber = await client.declareConsumer(
    {
      stream: "dynamic-test",
      offset: rabbit.Offset.next(),
      filter: {
        values,
        postFilterFunc(msg) {
          if (values.includes(msg.applicationProperties!.i as string))
            return true;
          else return false;
        },
        matchUnfiltered: false,
      },
    },
    fn(0)
    // (msg) => {
    //   console.log(msg);
    // }
  );
}

await createListener();
await publishAll();
