use futures_lite::StreamExt;
use lapin::{
    options::{BasicAckOptions, BasicConsumeOptions, QueueDeclareOptions},
    types::{AMQPValue, FieldArray, FieldTable},
    Connection, ConnectionProperties,
};

#[tokio::main]
async fn main() {
    let addr = std::env::var("AMQP_ADDR").unwrap_or_else(|_| "amqp://127.0.0.1:5672/%2f".into());

    dbg!("Connecting");
    let conn = Connection::connect(&addr, ConnectionProperties::default())
        .await
        .unwrap();

    dbg!("Create channel");
    let conn = conn.create_channel().await.unwrap();
    let mut args: FieldTable = Default::default();
    args.insert(
        "x-queue-type".into(),
        AMQPValue::LongString("stream".into()),
    );
    args.insert(
        "x-max-length-bytes".into(),
        AMQPValue::LongLongInt(5_000_000_000),
    );

    conn.queue_declare(
        "dynamic-test",
        QueueDeclareOptions {
            durable: true,
            ..Default::default()
        },
        args,
    )
    .await
    .unwrap();

    conn.basic_qos(100, Default::default()).await.unwrap();

    let mut args: FieldTable = Default::default();
    // args.insert("x-stream-offset", "...");

    let mut filter: FieldArray = Default::default();
    filter.push(AMQPValue::LongString("my-topic".into()));
    args.insert("x-stream-filter".into(), AMQPValue::FieldArray(filter));

    dbg!("Create consumer");
    let mut consumer = conn
        .basic_consume(
            "dynamic-test",
            Default::default(),
            BasicConsumeOptions {
                ..Default::default()
            },
            args,
        )
        .await
        .unwrap();

    // spawn(async move {
    // info!("will consume");
    dbg!("Listening...");
    while let Some(delivery) = consumer.next().await {
        let delivery = delivery.expect("error in consumer");
        delivery.ack(BasicAckOptions::default()).await.expect("ack");
        println!("Delivery! {:?}", delivery);
    }
    // })
    // .detach();
}
