use futures_lite::StreamExt;
use lapin::{
    options::{BasicAckOptions, BasicConsumeOptions, BasicPublishOptions, QueueDeclareOptions},
    protocol::basic::AMQPProperties,
    types::{AMQPValue, FieldArray, FieldTable},
    Connection, ConnectionProperties,
};

#[tokio::main]
async fn main() {
    let addr = std::env::var("AMQP_ADDR").unwrap_or_else(|_| "amqp://127.0.0.1:5672/%2f".into());

    dbg!("Connecting");
    let conn = Connection::connect(&addr, ConnectionProperties::default())
        .await
        .unwrap();

    dbg!("Create channel");
    let conn = conn.create_channel().await.unwrap();
    let mut args: FieldTable = Default::default();
    args.insert(
        "x-queue-type".into(),
        AMQPValue::LongString("stream".into()),
    );
    args.insert(
        "x-max-length-bytes".into(),
        AMQPValue::LongLongInt(5_000_000_000),
    );

    conn.queue_declare(
        "dynamic-test",
        QueueDeclareOptions {
            durable: true,
            ..Default::default()
        },
        args,
    )
    .await
    .unwrap();

    let mut headers: FieldTable = Default::default();
    headers.insert(
        "x-stream-filter-value".into(),
        AMQPValue::LongString("my-topic".into()),
    );

    let properties: AMQPProperties = Default::default();

    dbg!("Publishing...");
    conn.basic_publish(
        "",
        "dynamic-test",
        BasicPublishOptions {
            ..Default::default()
        },
        "helo".as_bytes(),
        properties.with_headers(headers),
    )
    .await
    .unwrap();
}
